"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var pdfJs = require("pdfjs-dist");
var fs = require("fs");
var path = require("path");
pdfJs.cMapUrl = path.join(require.resolve("pdfjs-dist"), "../../cmaps");
pdfJs.cMapPacked = true;
var CMapCompressionType = {
    NONE: 0,
    BINARY: 1,
    STREAM: 2
};
var NodeCMapReaderFactory = /** @class */ (function () {
    function NodeCMapReaderFactory() {
        this.baseUrl = path.join(require.resolve("pdfjs-dist"), "../../cmaps/");
        this.isCompressed = true;
    }
    NodeCMapReaderFactory.prototype.fetch = function (_a) {
        var _this = this;
        var name = _a.name;
        if (!name) {
            return Promise.reject(new Error("CMap name must be specified."));
        }
        return new Promise(function (resolve, reject) {
            var url = _this.baseUrl + name + (_this.isCompressed ? ".bcmap" : "");
            var fs = require("fs");
            fs.readFile(url, function (error, data) {
                if (error || !data) {
                    reject(new Error("Unable to load " + (_this.isCompressed ? "binary " : "") + "CMap at: " + url));
                    return;
                }
                resolve({
                    cMapData: new Uint8Array(data),
                    compressionType: _this.isCompressed ? CMapCompressionType.BINARY : CMapCompressionType.NONE
                });
            });
        });
    };
    return NodeCMapReaderFactory;
}());
//  Based on https://github.com/nemanjan00/search/blob/9919cf648194524f4811f32e57d036ef880dfedf/pdf.js
var openPDF = function (filePath) {
    return new Promise(function (resolve, reject) {
        fs.readFile(filePath, function (error, data) {
            if (error) {
                return reject(error);
            }
            var unitArray = new Uint8Array(data);
            pdfJs
                .getDocument({
                data: unitArray,
                CMapReaderFactory: NodeCMapReaderFactory
            })
                .then(function (pdf) { return resolve(pdf); });
        });
    });
};
var readPage = function (pdf, pageNumber) {
    return new Promise(function (resolve, reject) {
        pdf.getPage(pageNumber).then(function (page) {
            return page.getTextContent().then(function (page) {
                var items = page.items;
                var texts = items.map(function (item) {
                    return item.str;
                });
                resolve({
                    pageNumber: pageNumber,
                    texts: texts
                });
            }, reject);
        }, reject);
    });
};
var readAllPages = function (pdf) {
    var pages = [];
    for (var i = 0; i < pdf.numPages; i++) {
        pages.push(readPage(pdf, i + 1));
    }
    return Promise.all(pages);
};
exports.pdfToJSON = function (filePath) {
    return new Promise(function (resolve, reject) {
        openPDF(filePath)
            .then(function (pdf) {
            return readAllPages(pdf).then(function (pages) {
                pdf.getMetadata().then(function (meta) {
                    resolve({
                        meta: {
                            PDFFormatVersion: meta.info.PDFFormatVersion,
                            Title: meta.info.Title,
                            Author: meta.info.Author,
                            Creator: meta.info.Creator,
                            Producer: meta.info.Producer,
                            CreationDate: meta.info.CreationDate
                        },
                        filePath: filePath,
                        pages: pages
                    });
                }, reject);
            });
        })
            .catch(reject);
    });
};
//# sourceMappingURL=pdf-to-json.js.map